@@ -674,6 +674,7 @@ classdef calendarDuration
 ##                             Available Methods                              ##
 ##                                                                            ##
 ## 'size'             'ndims'            'numel'            'nnz'             ##
+## 'length'           'keyHash'                                               ##
 ##                                                                            ##
 ################################################################################
 
@@ -747,11 +748,11 @@ classdef calendarDuration
     endfunction
 
     ## -*- texinfo -*-
-    ## @deftypefn {calendarDuration} {@var{out} =} nns (@var{calD})
+    ## @deftypefn {calendarDuration} {@var{out} =} nnz (@var{calD})
     ##
     ## Number of nonzero elements in calendarDuration array.
     ##
-    ## @code{@var{out} = nns (@var{calD})} returns the number of nonzero
+    ## @code{@var{out} = nnz (@var{calD})} returns the number of nonzero
     ## elements in the calendar duration array @var{calD}.
     ##
     ## @end deftypefn
@@ -762,6 +763,65 @@ classdef calendarDuration
       out = numel (m) - sum (m == 0 & d == 0 & h == 0);
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{N} =} length (@var{calD})
+    ##
+    ## Length of a calendarDuration vector.
+    ##
+    ## @code{@var{N} = length (@var{calD})} returns the size of the longest
+    ## dimension of the calendarDuration array @var{calD}, unless any of its
+    ## dimensions has zero length, in which case @code{length (@var{calD})}
+    ## returns 0.
+    ##
+    ## @end deftypefn
+    function N = length (this)
+      if (isempty (this.Months))
+        N = 0;
+      else
+        N = max (size (this.Months));
+      endif
+    endfunction
+
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{hey} =} keyHash (@var{calD})
+    ##
+    ## Generate a hash code for a calendarDuration array.
+    ##
+    ## @code{@var{h} = keyHash (@var{calD})} generates a @qcode{uint64} scalar
+    ## that represents the input array @var{calD}.  @code{keyHash} utilizes the
+    ## 64-bit FMV-1a variant of the Fowler-Noll-Vo non-cryptographic hash
+    ## function.
+    ##
+    ## @code{@var{h} = keyHash (@var{calD}), @var{base}} also generates a 64-bit
+    ## hash code using @var{base} as the offset basis for the FNV-1a hash
+    ## algorithm.  @var{base} must be a @qcode{uint64} integer type scalar.  Use
+    ## this syntax to cascade @code{keyHash} on multiple objects for which a
+    ## single hash code is required.
+    ##
+    ## Note that unlike MATLAB, this implementation does no use any random seed.
+    ## As a result, @code{keyHash} will always generate the exact same hash key
+    ## for any particular input across different workers and Octave sessions.
+    ##
+    ## @end deftypefn
+    function key = keyHash (this, base = [])
+      ## Initialize string with size and class name
+      size_str = sprintf ('%dx', size (this.Months))(1:end-1);
+      init_str = [size_str 'calendarDuration'];
+      if (base)
+        if (! (isscalar (base) && isa (base, 'uint64')))
+          error ("calendarDuration.keyHash: BASE must be a UINT64 scalar.");
+        endif
+        key = __ckeyHash__(init_str, base);
+      else
+        key = __ckeyHash__(init_str);
+      endif
+      if (! isempty (this.Months))
+        key = __nkeyHash__(this.Months(:), key);
+        key = __nkeyHash__(this.Days(:), key);
+        key = keyHash (this.Time, key);
+      endif
+    endfunction
+
   endmethods
 
 ################################################################################
@@ -1016,6 +1076,21 @@ classdef calendarDuration
 
   methods (Access = public)
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} minus (@var{A}, @var{B})
+    ##
+    ## Subtraction for calendarDuration arrays.
+    ##
+    ## @code{@var{C} = minus (@var{A}, @var{B})} is the equivalent of the syntax
+    ## @code{@var{C} = @var{A} - @var{B}} and returns the result of substracting
+    ## the corresponding elements of @var{B} from those of @var{A}.  @var{C} is
+    ## a calendarDuration array of the same size as the input arguments after
+    ## the necessary (if required) expansion.  @var{A} and @var{B} must be size
+    ## compatible, which translates to they can be the same size, one can be
+    ## scalar, or for every dimension, their dimension sizes must be equal or
+    ## one of them must be 1.
+    ##
+    ## @end deftypefn
     function out = minus (A, B)
       if (isa (A, 'calendarDuration') && isa (B, 'calendarDuration'))
         out = A;
@@ -1041,6 +1116,17 @@ classdef calendarDuration
       out = broadcastProperties (out);
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} minus (@var{A})
+    ##
+    ## Unary minus for calendarDuration arrays.
+    ##
+    ## @code{@var{C} = uminus (@var{A})} is the equivalent of the syntax
+    ## @code{@var{C} = -@var{A}} and returns @var{A} after negating its
+    ## elements.  @var{C} is a calendarDuration array of the same size as
+    ## @var{A}.
+    ##
+    ## @end deftypefn
     function out = uminus (A)
       out = A;
       out.Months = -A.Months;
@@ -1048,6 +1134,21 @@ classdef calendarDuration
       out.Time = -A.Time;
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} plus (@var{A}, @var{B})
+    ##
+    ## Addition for calendarDuration arrays.
+    ##
+    ## @code{@var{C} = plus (@var{A}, @var{B})} is the equivalent of the syntax
+    ## @code{@var{C} = @var{A} + @var{B}} and returns the result of adding the
+    ## corresponding elements of @var{A} and @var{A}.  @var{C} is a
+    ## calendarDuration array of the same size as the input arguments after the
+    ## necessary (if required) expansion.  @var{A} and @var{B} must be size
+    ## compatible, which translates to they can be the same size, one can be
+    ## scalar, or for every dimension, their dimension sizes must be equal or
+    ## one of them must be 1.
+    ##
+    ## @end deftypefn
     function out = plus (A, B)
       if (isa (A, 'calendarDuration') && isa (B, 'calendarDuration'))
         out = A;
@@ -1073,6 +1174,16 @@ classdef calendarDuration
       endif
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} uplus (@var{A})
+    ##
+    ## Unary plus for calendarDuration arrays.
+    ##
+    ## @code{@var{C} = uminus (@var{A})} is the equivalent of the syntax
+    ## @code{@var{C} = +@var{A}} and returns a copy of @var{A}.  @var{C} is a
+    ## calendarDuration array of the same size as @var{A}.
+    ##
+    ## @end deftypefn
     function out = uplus (A)
       out = A;
       out.Months = A.Months;
@@ -1087,12 +1198,12 @@ classdef calendarDuration
 ################################################################################
 ##                             Available Methods                              ##
 ##                                                                            ##
-## 'sort'             'sortrows'         'unique'           'interp1'         ##
-## 'intersect'        'setdiff'          'setxor'           'union'           ##
+## 'sort'             'sortrows'         'unique'           'intersect'       ##
+## 'setdiff'          'setxor'           'union'                              ##
 ##                                                                            ##
 ################################################################################
 
-  methods (Access = public)
+  methods (Hidden)
 
     function [B, index] = sort (A, varargin)
       error ("calendarDuration.sort: not implemented yet.");
@@ -1106,10 +1217,6 @@ classdef calendarDuration
       error ("calendarDuration.unique: not implemented yet.");
     endfunction
 
-    function [C, ixA, ixB] = interp1 (A, B, varargin)
-      error ("calendarDuration.interp1: not implemented yet.");
-    endfunction
-
     function [C, ixA, ixB] = intersect (A, B, varargin)
       error ("calendarDuration.intersect: not implemented yet.");
     endfunction
@@ -1134,13 +1241,27 @@ classdef calendarDuration
 ##                             Available Methods                              ##
 ##                                                                            ##
 ## 'cat'              'horzcat'          'vertcat'          'repmat'          ##
-## 'reshape'          'circshift'        'permute'          'ipermute'        ##
-## 'transpose'        'ctranspose'                                            ##
+## 'repelem'          'repelems'         'reshape'          'circshift'       ##
+## 'permute'          'ipermute'         'transpose'        'ctranspose'      ##
 ##                                                                            ##
 ################################################################################
 
-  methods (Hidden)
+  methods (Access = public)
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} cat (@var{dim}, @var{A}, @var{B}, @dots{})
+    ##
+    ## Concatenate calendarDuration arrays.
+    ##
+    ## @code{@var{C} = cat (@var{dim}, @var{A}, @var{B}, @dots{})} concatenates
+    ## calendarDuration arrays @var{A}, @var{B}, @dots{} along dimension
+    ## @var{dim}.  All input arrays must have the same size except along the
+    ## operating dimension @var{dim}.  Any of the input arrays may also be
+    ## string arrays or cell arrays of character vectors of compatible size.
+    ## Additionally, an input can be a numeric matrix, which when parsed to the
+    ## constructor will return a calendarDuration array of compatible size.
+    ##
+    ## @end deftypefn
     function out = cat (dim, varargin)
       args = varargin;
       [args{:}] = promote (varargin{:});
@@ -1154,50 +1275,244 @@ classdef calendarDuration
       out = broadcastProperties (out);
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} horzcat (@var{A}, @var{B}, @dots{})
+    ##
+    ## Horizontal concatenation of calendarDuration arrays.
+    ##
+    ## @code{@var{C} = horzcat (@var{A}, @var{B}, @dots{}} is the equivalent of
+    ## the syntax @code{@var{B} = [@var{A}, @var{B}, @dots{}]} and horizontally
+    ## concatenates the calendarDuration arrays @var{A}, @var{B}, @dots{}.  All
+    ## input arrays must have the same size except along the second dimension.
+    ## Any of the input arrays may also be string arrays or cell arrays of
+    ## character vectors of compatible size.  Additionally, an input can be a
+    ## numeric matrix, which when parsed to the constructor will return a
+    ## calendarDuration array of compatible size.
+    ##
+    ## @end deftypefn
     function out = horzcat (varargin)
       out = cat (2, varargin{:});
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{C} =} vertat (@var{A}, @var{B}, @dots{})
+    ##
+    ## Vertical concatenation of calendarDuration arrays.
+    ##
+    ## @code{@var{C} = vertat (@var{A}, @var{B}, @dots{}} is the equivalent of
+    ## the syntax @code{@var{B} = [@var{A}; @var{B}; @dots{}]} and vertically
+    ## concatenates the calendarDuration arrays @var{A}, @var{B}, @dots{}.  All
+    ## input arrays must have the same size except along the first dimension.
+    ## Any of the input arrays may also be string arrays or cell arrays of
+    ## character vectors of compatible size.  Additionally, an input can be a
+    ## numeric matrix, which when parsed to the constructor will return a
+    ## calendarDuration array of compatible size.
+    ##
+    ## @end deftypefn
     function out = vertcat (varargin)
       out = cat (1, varargin{:});
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn  {calendarDuration} {@var{B} =} repmat (@var{A}, @var{n})
+    ## @deftypefnx {calendarDuration} {@var{B} =} repmat (@var{A}, @var{d1}, @dots{}, @var{dN})
+    ## @deftypefnx {calendarDuration} {@var{B} =} repmat (@var{A}, @var{dimvec})
+    ##
+    ## Repeat copies of a calendarDuration array.
+    ##
+    ## @code{@var{B} = repmat (@var{A}, @var{n})} returns a calendarDuration
+    ## array @var{B} containing @var{n} copies of the input calendarDuration
+    ## array @var{A} along every dimension of @var{A}.
+    ##
+    ## @code{@var{B} = repmat (@var{A}, @var{d1}, @dots{}, @var{dN})} returns an
+    ## array @var{B} containing copies of @var{A} along the dimensions specified
+    ## by the list of scalar integer values @var{d1}, @dots{}, @var{dN}, which
+    ## specify how many copies of @var{A} are made in each dimension.
+    ##
+    ## @code{@var{B} = repmat (@var{A}, @var{dimvec})} is equivalent to the
+    ## previous syntax with @code{@var{dimvec} = [@var{d1}, @dots{}, @var{dN}]}.
+    ##
+    ## @end deftypefn
     function this = repmat (this, varargin)
       this.Months = repmat (this.Months, varargin{:});
       this.Days   = repmat (this.Days, varargin{:});
       this.Time   = repmat (this.Time, varargin{:});
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn  {calendarDuration} {@var{B} =} repelem (@var{A}, @var{n})
+    ## @deftypefnx {calendarDuration} {@var{B} =} repelem (@var{A}, @var{d1}, @dots{}, @var{dN})
+    ##
+    ## Repeat copies of calendarDuration array elements.
+    ##
+    ## @code{@var{B} = repelem (@var{A}, @var{n})} returns a calendarDuration
+    ## vector @var{B} containing repeated elements of the input @var{A}, which
+    ## must be a calendarDuration vector.  If @var{n} is a scalar, each element
+    ## of @var{A} is repeated @var{n} times along the non-singleton dimension of
+    ## @var{A}.  If @var{n} is a vector, it must have the same elemnts as
+    ## @var{A}, in which case it specifies the number of times to repeat each
+    ## corresponding element of @var{A}.
+    ##
+    ## @code{@var{B} =} repelem (@var{A}, @var{d1}, @dots{}, @var{dN}} returns
+    ## an array @var{B} with each element of @var{A} repeated according to the
+    ## the list of input arguments @code{@var{d1}, @dots{}, @var{dN}} each
+    ## corresponding to a different dimension @code{1:ndims (@var{A})} of the
+    ## input array @var{A}.  @var{d1}, @dots{}, @var{dN} must be either scalars
+    ## or vectors with the same length as the corresponding dimension of
+    ## @var{A} containing non-negative integer values specifying the number of
+    ## repetitions of each element along the corresponding dimension.
+    ##
+    ## @end deftypefn
+    function this = repelem (this, varargin)
+      this.Months = repelem (this.Months, varargin{:});
+      this.Days   = repelem (this.Days, varargin{:});
+      this.Time   = repelem (this.Time, varargin{:});
+    endfunction
+
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{B} =} repelems (@var{A}, @var{R})
+    ##
+    ## Construct a vector of repeated elements from a calendarDuration array.
+    ##
+    ## @code{@var{B} = repelems (@var{A}, @var{R})} returns a calendarDuration
+    ## vector @var{B} containing repeated elements of the input @var{A}, which
+    ## must be a calendarDuration vector.  @var{R} must be a @math{2xN} matrix
+    ## of integers.  Entries in the first row of @var{R} correspond to the
+    ## linear indexing of the elements in @var{A} to be repeated.  The
+    ## corresponding entries in the second row of @var{R} specify the repeat
+    ## count of each element.
+    ##
+    ## @end deftypefn
+    function this = repelems (this, R)
+      this.Months = repelems (this.Months, R);
+      this.Days   = repelems (this.Days, R);
+      this.Time   = repelems (this.Time, R);
+    endfunction
+
+    ## -*- texinfo -*-
+    ## @deftypefn  {calendarDuration} {@var{B} =} reshape (@var{A}, @var{d1}, @dots{}, @var{dN})
+    ## @deftypefnx {calendarDuration} {@var{B} =} reshape (@var{A}, @dots{}, @qcode{[]}, @dots{})
+    ## @deftypefnx {calendarDuration} {@var{B} =} reshape (@var{A}, @var{dimvec})
+    ##
+    ## Repeat copies of calendarDuration array elements.
+    ##
+    ## @code{@var{B} = reshape (@var{A}, @var{d1}, @dots{}, @var{dN})} returns a
+    ## calendarDuration array @var{B} with specified dimensions @var{d1},
+    ## @dots{}, @var{dN}, whose elements are taken columnwise from the
+    ## calendarDuration array @var{A}.  The product of @var{d1}, @dots{},
+    ## @var{dN} must equal the total number of elements in @var{A}.
+    ##
+    ## @code{@var{B} = reshape (@var{A}, @dots{}, @qcode{[]}, @dots{})} returns
+    ## a calendarDuration array @var{B} with one dimension unspecified which is
+    ## calculated automatically so that the product of dimensions in @var{B}
+    ## matches the total elements in @var{A}, which must be divisible the
+    ## product of specified dimensions.  An empty matrix @qcode{([])} is used to
+    ## flag the unspecified dimension.
+    ##
+    ## @end deftypefn
     function this = reshape (this, varargin)
       this.Months = reshape (this.Months, varargin{:});
       this.Days   = reshape (this.Days, varargin{:});
       this.Time   = reshape (this.Time, varargin{:});
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn  {calendarDuration} {@var{B} =} circshift (@var{A}, @var{n})
+    ## @deftypefnx {calendarDuration} {@var{B} =} circshift (@var{A}, @var{n}, @var{dim})
+    ##
+    ## Circularly shift the elements in a calendarDuration array.
+    ##
+    ## @code{@var{B} = circshift (@var{A}, @var{n})} circularly shifts the
+    ## elements of the calendarDuration array @var{A} according to @var{n}.  If
+    ## @var{n} is a nonzero integer scalar, then the elements of @var{A} are
+    ## shifted by @var{n} elements along the first non-singleton dimension of
+    ## @var{A}.  If @var{n} is a vector, it must not be longer that the number
+    ## of dimensions of @var{A} with each value of @var{n} corresponding to a
+    ## dimension in @var{A}.   The sign of the value(s) in @var{n} specify the
+    ## direction in the elements of @var{A} are shifted.
+    ##
+    ## @code{@var{B} = circshift (@var{A}, @var{n}, @var{dim})} circularly
+    ## shifts the elements of the calendarDuration array @var{A} along the
+    ## dimension specified by @var{dim}.  In this case, @var{n} must be a scalar
+    ## integer value.
+    ##
+    ## @end deftypefn
     function this = circshift (this, varargin)
       this.Months = circshift (this.Months, varargin{:});
       this.Days   = circshift (this.Days, varargin{:});
       this.Time   = circshift (this.Time, varargin{:});
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{B} =} permute (@var{A}, @var{dims})
+    ##
+    ## Generalized transpose for a calendarDuration N-D array.
+    ##
+    ## @code{@var{B} = permute (@var{A}, @var{dims})} returns the generalized
+    ## transpose of the calendarDuration array @var{A} by rearranging its
+    ## dimensions according to the permutation vector specified in @var{dims}.
+    ##
+    ## @var{dims} must index all the dimensions @code{1:ndims (@var{A})} of the
+    ## input array @var{A}, in any order, but only once.  The @var{N}th
+    ## dimension of @var{A} gets remapped to the dimension in @var{B} specified
+    ## by @code{@var{dims}(@var{N})}.
+    ##
+    ## @end deftypefn
     function this = permute (this, order)
       this.Months = permute (this.Months, order);
       this.Days   = permute (this.Days, order);
       this.Time   = permute (this.Time, order);
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{A} =} ipermute (@var{B}, @var{dims})
+    ##
+    ## Inverse of the generalized transpose for a calendarDuration N-D array.
+    ##
+    ## @code{@var{A} = ipermute (@var{B}, @var{dims})} returns the inverse of
+    ## the generalized transpose performed by the @code{permute} function.  The
+    ## expression @code{ipermute (permute (@var{A}, @var{dims}), @var{dims})}
+    ## returns the original array @var{A}.
+    ##
+    ## @var{dims} must index all the dimensions @code{1:ndims (@var{B})} of the
+    ## input array @var{B}, in any order, but only once.  The dimension of
+    ## @var{B} specified in @code{@var{dims}(@var{N})} gets remapped to the
+    ## @var{N}th dimension of @var{A}.
+    ##
+    ## @end deftypefn
     function this = ipermute (this, order)
       this.Months = ipermute (this.Months, order);
       this.Days   = ipermute (this.Days, order);
       this.Time   = ipermute (this.Time, order);
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{B} =} transpose (@var{A})
+    ##
+    ## Transpose a calendarDuration matrix.
+    ##
+    ## @code{@var{B} = transpose (@var{A})} is the equivalent of the syntax
+    ## @code{@var{B} = @var{A}.'} and returns the transpose of the
+    ## calendarDuration matrix @var{A}.
+    ##
+    ## @end deftypefn
     function this = transpose (this)
       this.Months = transpose (this.Months);
       this.Days   = transpose (this.Days);
       this.Time   = transpose (this.Time);
     endfunction
 
+    ## -*- texinfo -*-
+    ## @deftypefn {calendarDuration} {@var{B} =} ctranspose (@var{A})
+    ##
+    ## Transpose a calendarDuration matrix.
+    ##
+    ## @code{@var{B} = ctranspose (@var{A})} is the equivalent of the syntax
+    ## @code{@var{B} = @var{A}'} and returns the transpose of the
+    ## calendarDuration matrix @var{A}.  For calendarDuration arrays,
+    ## @code{ctranspose} is identical to @code{transpose}.
+    ##
+    ## @end deftypefn
     function this = ctranspose (this)
       this.Months = ctranspose (this.Months);
       this.Days   = ctranspose (this.Days);
@@ -1277,12 +1592,27 @@ classdef calendarDuration
       endif
       switch s.type
         case '()'
-          if (! isa (val, "calendarDuration"))
-            val = calendarDuration (val);
+          if (isempty (val))
+            this.Months(s.subs{:}) = [];
+            this.Days(s.subs{:})   = [];
+            this.Time(s.subs{:})   = [];
+            return;
+          elseif (isnumeric (val))
+            tmp = zeros (size (val));
+            this.Months(s.subs{:}) = tmp;
+            this.Days(s.subs{:})   = tmp;
+            this.Time(s.subs{:})   = duration (24 * double (val), 0, 0);
+            this = broadcastProperties (this);
+          elseif (isa (val, "calendarDuration"))
+            this.Months(s.subs{:}) = val.Months;
+            this.Days(s.subs{:})   = val.Days;
+            this.Time(s.subs{:})   = val.Time;
+            this = broadcastProperties (this);
+          else
+            error (strcat ("calendarDuration.subsasgn: assignment value", ...
+                           " must be calendarDuration array or a numeric", ...
+                           " array representing 24-hour days."));
           endif
-          this.Months(s.subs{:}) = val.Months;
-          this.Days(s.subs{:})   = val.Days;
-          this.Time(s.subs{:})   = val.Time;
 
         case '{}'
           error (["calendarDuration.subsasgn: '{}' invalid indexing", ...
@@ -1312,6 +1642,14 @@ classdef calendarDuration
 
   methods (Access = private)
 
+    ## Return a subset of the array
+    function this = subset (this, varargin)
+      this = this;
+      this.Months = this.Months(varargin{:});
+      this.Days   = this.Days(varargin{:});
+      this.Time   = this.Time(varargin{:});
+    endfunction
+
     ## Broadcast properties
     function this = broadcastProperties (this)
       ## Handle NaNs and Infs first
@@ -1377,14 +1715,6 @@ classdef calendarDuration
       endif
     endfunction
 
-    ## Return a subset of the array
-    function this = subset (this, varargin)
-      this = this;
-      this.Months = this.Months(varargin{:});
-      this.Days   = this.Days(varargin{:});
-      this.Time   = this.Time(varargin{:});
-    endfunction
-
   endmethods
 
 endclassdef
